@@ -58,6 +58,7 @@ from ._partition import PartitionManager
 from ._activation_profile import ActivationProfileManager
 from ._adapter import AdapterManager
 from ._virtual_switch import VirtualSwitchManager
+from ._capacity_group import CapacityGroupManager
 from ._logging import logged_api_call
 from ._exceptions import ParseError
 
@@ -230,6 +231,7 @@ class Cpc(BaseResource):
         self._partitions = None
         self._adapters = None
         self._virtual_switches = None
+        self._capacity_groups = None
         self._reset_activation_profiles = None
         self._image_activation_profiles = None
         self._load_activation_profiles = None
@@ -278,6 +280,17 @@ class Cpc(BaseResource):
             self._virtual_switches = VirtualSwitchManager(self)
         return self._virtual_switches
 
+    @property
+    def capacity_groups(self):
+        """
+        :class:`~zhmcclient.CapacityGroupManager`: Access to the
+        :term:`Capacity Groups <Capacity Group>` in this CPC.
+        """
+        # We do here some lazy loading.
+        if not self._capacity_groups:
+            self._capacity_groups = CapacityGroupManager(self)
+        return self._capacity_groups
+
     @property
     def vswitches(self):
         """
@@ -356,6 +369,12 @@ class Cpc(BaseResource):
         """
         return self.prop('dpm-enabled', False)
 
+    # Note: From HMC API version 2.24 on (i.e. starting with 8561), the Cpc
+    # object supports a 'maximum-partitions' property, but only in DPM mode.
+    # Therefore, we need to continue maintaining max partitions for all future
+    # machine types.
+
+    # Machine types with same max partitions for all models:
     _MAX_PARTITIONS_BY_MACHINE_TYPE = {
         '2817': 60,  # z196
         '2818': 30,  # z114
@@ -365,11 +384,15 @@ class Cpc(BaseResource):
         '2965': 40,  # z13s / Rockhopper
         '3906': 85,  # z14 / Emperor II
         '3907': 40,  # z14-ZR1 / Rockhopper II
-        # Note: From HMC API version 2.24 on, the Cpc object supports a
-        # 'maximum-partitions' property. Because that API version was
-        # introduced while model 3907 was already avbailable, the property is
-        # guaranteed to be available only on models after 3907.
-        # TODO: Exploit the new 'maximum-partitions' property.
+    }
+
+    # Machine types with different max partitions across their models:
+    _MAX_PARTITIONS_BY_MACHINE_TYPE_MODEL = {
+        ('8561', 'T01'): 85,  # z15
+        ('8561', 'LT1'): 85,  # z15
+        ('8562', 'GT2'): 85,  # z15 (85 is an exception for 8562)
+        ('8562', 'T02'): 40,  # z15
+        ('8562', 'LT2'): 40,  # z15
     }
 
     @property
@@ -393,7 +416,9 @@ class Cpc(BaseResource):
         z13 / Emperor                             85
         z13s / Rockhopper                         40
         z14 / Emperor II                          85
-        z14-ZR1 / Rockhopper II                   40
+        z14-ZR1 / -LR1                            40
+        z15-T01 / -LT1 / -GT2                     85
+        z15-T02 / -LT2                            40
         =========================  ==================
 
         Raises:
@@ -405,11 +430,23 @@ class Cpc(BaseResource):
           :exc:`ValueError`: Unknown machine type
         """
         machine_type = self.get_property('machine-type')
+        machine_model = self.get_property('machine-model')
         try:
-            max_parts = self._MAX_PARTITIONS_BY_MACHINE_TYPE[machine_type]
+            return self._MAX_PARTITIONS_BY_MACHINE_TYPE[machine_type]
         except KeyError:
-            raise ValueError("Unknown machine type: {!r}".format(machine_type))
-        return max_parts
+            pass
+        try:
+            return self._MAX_PARTITIONS_BY_MACHINE_TYPE_MODEL[
+                (machine_type, machine_model)]
+        except KeyError:
+            pass
+        try:
+            return self.get_property('maximum-partitions')
+        except KeyError:
+            new_exc = ValueError("Unknown machine type/model: {}-{}".
+                                 format(machine_type, machine_model))
+            new_exc.__cause__ = None
+            raise new_exc  # ValueError
 
     @logged_api_call
     def feature_enabled(self, feature_name):
@@ -455,7 +492,7 @@ class Cpc(BaseResource):
         else:
             raise ValueError("Firmware feature %s is not available on CPC %s" %
                              (feature_name, self.name))
-        return feature['state']
+        return feature['state']  # pylint: disable=undefined-loop-variable
 
     @logged_api_call
     def feature_info(self):
@@ -518,6 +555,7 @@ class Cpc(BaseResource):
           :exc:`~zhmcclient.AuthError`
           :exc:`~zhmcclient.ConnectionError`
         """
+        # pylint: disable=protected-access
         self.manager.session.post(self.uri, body=properties)
         # Attempts to change the 'name' property will be rejected by the HMC,
         # so we don't need to update the name-to-URI cache.
@@ -1337,3 +1375,182 @@ class Cpc(BaseResource):
         self.manager.session.post(
             self.uri + '/operations/validate-lun-path',
             body=body)
+
+    @logged_api_call
+    def add_temporary_capacity(
+            self, record_id, software_model=None, processor_info=None,
+            test=False, force=False):
+        """
+        Add temporary processors to the CPC or increase temporary model
+        capacity of the CPC.
+
+        This method performs the "Add Temporary Capacity" HMC operation.
+
+        If the request would exceed the processor capacity that is installed on
+        the CPC or the limits of the capacity record, the operation will fail,
+        unless the `force` parameter is `True`.
+
+        Parameters:
+
+          record_id (:term:`string`):
+            The ID of the capacity record to be used for any updates of the
+            processor capacity.
+
+          software_model (:term:`string`):
+            The name of the software model to be activated for the CPC.
+            This must be one of the software models defined within the
+            specified capacity record. The software model implies the number
+            of general purpose processors that will be active once the
+            operation succeeds.
+
+            If `None`, the software model and the number of general purpose
+            processors of the CPC will remain unchanged.
+
+          processor_info (dict):
+            The number of specialty processors to be added to the CPC.
+
+            If `None`, the number of specialty processors of the CPC will
+            remain unchanged.
+
+            Each item in the dictionary identifies the number of one type of
+            specialty processor. The key of the item must be a string
+            specifying the type of specialty processor ('aap', 'cbp', 'icf',
+            'ifl', 'iip', 'sap'), and the value of the item must be an integer
+            specifying the number of processors of that type to be added.
+
+            If an item for a type of specialty processor is not provided, or
+            if the value of the item is `None`, the number of specialty
+            processors of that type will remain unchanged.
+
+          test (bool):
+            Indicates whether test (`True`) or real (`False`) resources should
+            be activated. Test resources are automatically deactivated after
+            24h.
+
+          force (bool):
+            `True` indicates that the operation should proceed if not enough
+            processors are available. `True` is permitted only for CBU, CPE
+            and loaner capacity records.
+
+        Authorization requirements:
+
+        * Object-access permission to the CPC.
+        * Task permission to the "Perform Model Conversion" task.
+
+        Raises:
+
+          :exc:`~zhmcclient.HTTPError`
+          :exc:`~zhmcclient.ParseError`
+          :exc:`~zhmcclient.AuthError`
+          :exc:`~zhmcclient.ConnectionError`
+        """
+
+        body = {
+            'record-id': record_id,
+            'force': force,
+            'test': test,
+        }
+        if software_model:
+            body['software-model'] = software_model
+        if processor_info:
+            pi = []
+            for ptype, pvalue in processor_info.items():
+                pi_item = {
+                    'processor-type': ptype,
+                }
+                if pvalue is not None:
+                    pi_item['num-processor-steps'] = pvalue
+                pi.append(pi_item)
+            body['processor-info'] = pi
+
+        self.manager.session.post(
+            self.uri + '/operations/add-temp-capacity',
+            body=body)
+
+    @logged_api_call
+    def remove_temporary_capacity(
+            self, record_id, software_model=None, processor_info=None):
+        """
+        Remove temporary processors from the CPC or decrease temporary model
+        capacity of the CPC.
+
+        This method performs the "Remove Temporary Capacity" HMC operation.
+
+        You can only remove activated resources for the specific offering.
+        You cannot remove dedicated engines or the last processor of a processor
+        type. If you remove resources back to the base configuration, the
+        capacity record activation is completed. That is, if you remove the
+        last temporary processor, your capacity record is deactivated. For a
+        CBU and On/Off CoD record, to add resources again, you must use another
+        :meth:`add_temporary_capacity` operation. For an On/Off CoD test or
+        CPE record, once the record is deactivated, it is no longer available
+        for use. You can then delete the record.
+
+        After removal of the resources, the capacity record remains as an
+        installed record. If you want a record deleted, you must manually
+        delete the record on the "Installed Records" page in the HMC GUI.
+
+        Parameters:
+
+          record_id (:term:`string`):
+            The ID of the capacity record to be used for any updates of the
+            processor capacity.
+
+          software_model (:term:`string`):
+            The name of the software model to be activated for the CPC.
+            This must be one of the software models defined within the
+            specified capacity record. The software model implies the number
+            of general purpose processors that will be active once the
+            operation succeeds.
+
+            If `None`, the software model and the number of general purpose
+            processors of the CPC will remain unchanged.
+
+          processor_info (dict):
+            The number of specialty processors to be removed from the CPC.
+
+            If `None`, the number of specialty processors of the CPC will
+            remain unchanged.
+
+            Each item in the dictionary identifies the number of one type of
+            specialty processor. The key of the item must be a string
+            specifying the type of specialty processor ('aap', 'cbp', 'icf',
+            'ifl', 'iip', 'sap'), and the value of the item must be an integer
+            specifying the number of processors of that type to be removed.
+
+            If an item for a type of specialty processor is not provided, or
+            if the value of the item is `None`, the number of specialty
+            processors of that type will remain unchanged.
+
+        Authorization requirements:
+
+        * Object-access permission to the CPC.
+        * Task permission to the "Perform Model Conversion" task.
+
+        Raises:
+
+          :exc:`~zhmcclient.HTTPError`
+          :exc:`~zhmcclient.ParseError`
+          :exc:`~zhmcclient.AuthError`
+          :exc:`~zhmcclient.ConnectionError`
+        """
+
+        body = {
+            'record-id': record_id,
+        }
+        if software_model:
+            body['software-model'] = software_model
+        if processor_info:
+            pi = []
+            for ptype, pvalue in processor_info.items():
+                pi_item = {
+                    'processor-type': ptype,
+                }
+                if pvalue is not None:
+                    pi_item['num-processor-steps'] = pvalue
+                pi.append(pi_item)
+            body['processor-info'] = pi
+
+        self.manager.session.post(
+            self.uri + '/operations/remove-temp-capacity',
+            body=body)
