@@ -22,7 +22,8 @@ import re
 __all__ = ['Error', 'ConnectionError', 'ConnectTimeout', 'ReadTimeout',
            'RetriesExceeded', 'AuthError', 'ClientAuthError',
            'ServerAuthError', 'ParseError', 'VersionError', 'HTTPError',
-           'OperationTimeout', 'StatusTimeout', 'NoUniqueMatch', 'NotFound']
+           'OperationTimeout', 'StatusTimeout', 'NoUniqueMatch', 'NotFound',
+           'MetricsResourceNotFound']
 
 
 class Error(Exception):
@@ -35,15 +36,6 @@ class Error(Exception):
     Derived from :exc:`~py:exceptions.Exception`.
     """
 
-    def __init__(self, *args):
-        # Parameters:
-        #   *args:
-        #     A list of input arguments for the exception object.
-        #     The derived classes define more specific parameters.
-        #     These input arguments will be available as tuple items in the
-        #     ``args`` instance variable of the exception object.
-        super(Error, self).__init__(*args)
-
     def str_def(self):
         """
         Interface definition for the corresponding method derived exception
@@ -59,6 +51,7 @@ class Error(Exception):
 
 
 class ConnectionError(Error):
+    # pylint: disable=redefined-builtin
     """
     This exception indicates a problem with the connection to the HMC, below
     the HTTP level. HTTP errors are indicated via :exc:`~zhmcclient.HTTPError`.
@@ -333,6 +326,7 @@ class RetriesExceeded(ConnectionError):
 
 
 class AuthError(Error):
+    # pylint: disable=abstract-method
     """
     This exception indicates erors related to authentication.
 
@@ -341,15 +335,7 @@ class AuthError(Error):
 
     Derived from :exc:`~zhmcclient.Error`.
     """
-
-    def __init__(self, *args):
-        # Parameters:
-        #   *args:
-        #     A list of input arguments for the exception object.
-        #     The derived classes define more specific parameters.
-        #     These input arguments will be available as tuple items in the
-        #     ``args`` instance variable of the exception object.
-        super(AuthError, self).__init__(*args)
+    pass
 
 
 class ClientAuthError(AuthError):
@@ -361,6 +347,7 @@ class ClientAuthError(AuthError):
     """
 
     def __init__(self, msg):
+        # pylint: disable=useless-super-delegation
         """
         Parameters:
 
@@ -440,6 +427,7 @@ class ServerAuthError(AuthError):
                       self.details.http_status, self.details.reason)
 
     def str_def(self):
+        # pylint: disable=line-too-long
         """
         :term:`string`: The exception as a string in a Python definition-style
         format, e.g. for parsing by scripts:
@@ -448,6 +436,7 @@ class ServerAuthError(AuthError):
 
             classname={}; request_method={}; request_uri={}; http_status={}; reason={}; message={};
         """  # noqa: E501
+        # pylint: enable=line-too-long
         return "classname={!r}; request_method={!r}; request_uri={!r}; " \
             "http_status={!r}; reason={!r}; message={!r};". \
             format(self.__class__.__name__, self.details.request_method,
@@ -775,9 +764,10 @@ class HTTPError(Error):
         """
         Return a human readable string representation of this exception object.
         """
-        return "{},{}: {} [{} {}]".\
+        stack_txt = ' stack={!r}'.format(self.stack) if self.stack else ''
+        return "{},{}: {} [{} {}]{}".\
                format(self.http_status, self.reason, self.message,
-                      self.request_method, self.request_uri)
+                      self.request_method, self.request_uri, stack_txt)
 
     def __repr__(self):
         """
@@ -785,11 +775,13 @@ class HTTPError(Error):
         purposes.
         """
         return "{}(http_status={!r}, reason={!r}, message={!r}, " \
-               "request_method={!r}, request_uri={!r}, ...)". \
+               "request_method={!r}, request_uri={!r}, stack={!r}, ...)". \
                format(self.__class__.__name__, self.http_status, self.reason,
-                      self.message, self.request_method, self.request_uri)
+                      self.message, self.request_method, self.request_uri,
+                      self.stack)
 
     def str_def(self):
+        # pylint: disable=line-too-long
         """
         :term:`string`: The exception as a string in a Python definition-style
         format, e.g. for parsing by scripts:
@@ -798,6 +790,7 @@ class HTTPError(Error):
 
             classname={}; request_method={}; request_uri={}; http_status={}; reason={}; message={};
         """  # noqa: E501
+        # pylint: enable=line-too-long
         return "classname={!r}; request_method={!r}; request_uri={!r}; " \
             "http_status={!r}; reason={!r}; message={!r};". \
             format(self.__class__.__name__, self.request_method,
@@ -938,6 +931,7 @@ class StatusTimeout(Error):
                    self.desired_statuses, self.status_timeout)
 
     def str_def(self):
+        # pylint: disable=line-too-long
         """
         :term:`string`: The exception as a string in a Python definition-style
         format, e.g. for parsing by scripts:
@@ -946,6 +940,7 @@ class StatusTimeout(Error):
 
             classname={}; actual_status={}; desired_statuses={}; status_timeout={}; message={};
         """  # noqa: E501
+        # pylint: enable=line-too-long
         return "classname={!r}; actual_status={!r}; desired_statuses={!r}; " \
             "status_timeout={!r}; message={!r};". \
             format(self.__class__.__name__, self.actual_status,
@@ -1049,6 +1044,7 @@ class NoUniqueMatch(Error):
                       self.resource_uris)
 
     def str_def(self):
+        # pylint: disable=line-too-long
         """
         :term:`string`: The exception as a string in a Python definition-style
         format, e.g. for parsing by scripts:
@@ -1057,6 +1053,7 @@ class NoUniqueMatch(Error):
 
             classname={}; resource_classname={}; filter_args={}; parent_classname={}; manager_name={}; message={}; resource_uris={}
         """  # noqa: E501
+        # pylint: enable=line-too-long
         parent = self.manager.parent
         return "classname={!r}; resource_classname={!r}; filter_args={!r}; " \
                "parent_classname={!r}; parent_name={!r}; message={!r}; " \
@@ -1145,6 +1142,7 @@ class NotFound(Error):
                       parent.name if parent else None)
 
     def str_def(self):
+        # pylint: disable=line-too-long
         """
         :term:`string`: The exception as a string in a Python definition-style
         format, e.g. for parsing by scripts:
@@ -1153,6 +1151,7 @@ class NotFound(Error):
 
             classname={}; resource_classname={}; filter_args={}; parent_classname={}; parent_name={}; message={};
         """  # noqa: E501
+        # pylint: enable=line-too-long
         parent = self.manager.parent
         return "classname={!r}; resource_classname={!r}; filter_args={!r}; " \
                "parent_classname={!r}; parent_name={!r}; message={!r};". \
@@ -1162,3 +1161,68 @@ class NotFound(Error):
                       parent.__class__.__name__ if parent else None,
                       parent.name if parent else None,
                       self.args[0])
+
+
+class MetricsResourceNotFound(Error):
+    # pylint: disable=redefined-builtin
+    """
+    This exception indicates that the resource referenced by a metric object
+    value was not found on the HMC.
+
+    Derived from :exc:`~zhmcclient.Error`.
+    """
+
+    def __init__(self, msg, resource_class, managers):
+        """
+        Parameters:
+
+          msg (:term:`string`):
+            A human readable message describing the problem.
+
+          resource_class (:class:`~zhmcclient.BaseResource`):
+            The zhmcclient resource class of the resource that was not found.
+
+          managers (list of :class:`~zhmcclient.BaseManager`):
+            List of zhmcclient resource managers that were searched for the
+            resource.
+
+        ``args[0]`` will be set to the ``msg`` parameter.
+        """
+        super(MetricsResourceNotFound, self).__init__(msg)
+        self._resource_class = resource_class
+        self._managers = managers
+
+    @property
+    def resource_class(self):
+        """
+        The zhmcclient resource class of the resource that was not found.
+        """
+        return self._resource_class
+
+    @property
+    def managers(self):
+        """
+        List of zhmcclient resource managers that were searched for the
+        resource
+        """
+        return self._managers
+
+    def __repr__(self):
+        """
+        Return a string with the state of this exception object, for debug
+        purposes.
+        """
+        return "{}(message={!r}". \
+               format(self.__class__.__name__, self.args[0])
+
+    def str_def(self):
+        """
+        :term:`string`: The exception as a string in a Python definition-style
+        format, e.g. for parsing by scripts:
+
+        .. code-block:: text
+
+            classname={}; message={}
+        """
+        return "classname={!r}; message={!r}". \
+            format(self.__class__.__name__, self.args[0])
