@@ -209,6 +209,7 @@ class Lpar(BaseResource):
           :exc:`~zhmcclient.AuthError`
           :exc:`~zhmcclient.ConnectionError`
         """
+        # pylint: disable=protected-access
         self.manager.session.post(self.uri, body=properties)
         # Attempts to change the 'name' property will be rejected by the HMC,
         # so we don't need to update the name-to-URI cache.
@@ -230,8 +231,9 @@ class Lpar(BaseResource):
         this method repeatedly checks the status of the LPAR after the HMC
         operation has completed, and waits until the status is in the desired
         state "not-operating" (which indicates that the LPAR is active but
-        no operating system is running), or if `allow_status_exceptions` was
-        set additionally in the state "exceptions".
+        no operating system is running), or "operating", or if
+        `allow_status_exceptions` was set additionally in the state
+        "exceptions".
 
         Authorization requirements:
 
@@ -247,8 +249,8 @@ class Lpar(BaseResource):
 
             * If `True`, this method will wait for completion of the
               asynchronous job performing the operation, and for the status
-              becoming "not-operating" (or in addition "exceptions", if
-              `allow_status_exceptions` was set.
+              becoming "not-operating" or "operating" (or in addition
+              "exceptions", if `allow_status_exceptions` was set.
 
             * If `False`, this method will return immediately once the HMC has
               accepted the request to perform the operation.
@@ -319,7 +321,7 @@ class Lpar(BaseResource):
             wait_for_completion=wait_for_completion,
             operation_timeout=operation_timeout)
         if wait_for_completion:
-            statuses = ["not-operating"]
+            statuses = ["not-operating", "operating"]
             if allow_status_exceptions:
                 statuses.append("exceptions")
             self.wait_for_status(statuses, status_timeout)
@@ -431,7 +433,9 @@ class Lpar(BaseResource):
                   operating_system_specific_load_parameters=None,
                   boot_record_logical_block_address=None, force=False,
                   wait_for_completion=True, operation_timeout=None,
-                  status_timeout=None, allow_status_exceptions=False):
+                  status_timeout=None, allow_status_exceptions=False,
+                  secure_boot=False):
+        # pylint: disable=invalid-name
         """
         Load (boot) this LPAR from a designated SCSI device, using the
         HMC operation "SCSI Load".
@@ -482,7 +486,7 @@ class Lpar(BaseResource):
 
           force (bool):
             Boolean controlling whether this operation is permitted when the
-            LPAR is in the "operating" status. The default value is `True`.
+            LPAR is in the "operating" status.
 
           wait_for_completion (bool):
             Boolean controlling whether this method should wait for completion
@@ -517,6 +521,11 @@ class Lpar(BaseResource):
             an additional acceptable end status when `wait_for_completion` is
             set.
 
+          secure_boot (bool):
+            Bollean controlling whether the system checks the software
+            signature of what is loaded against what the distributor signed it
+            with. Requires z15 or later.
+
         Returns:
 
           `None` or :class:`~zhmcclient.Job`:
@@ -554,6 +563,8 @@ class Lpar(BaseResource):
                 boot_record_logical_block_address
         if force:
             body['force'] = force
+        if secure_boot:
+            body['secure-boot'] = secure_boot
         result = self.manager.session.post(
             self.uri + '/operations/scsi-load',
             body,
@@ -572,7 +583,9 @@ class Lpar(BaseResource):
                   operating_system_specific_load_parameters=None,
                   boot_record_logical_block_address=None, os_ipl_token=None,
                   wait_for_completion=True, operation_timeout=None,
-                  status_timeout=None, allow_status_exceptions=False):
+                  status_timeout=None, allow_status_exceptions=False,
+                  force=False):
+        # pylint: disable=invalid-name
         """
         Load a standalone dump program from a designated SCSI device
         in this LPAR, using the HMC operation "SCSI Dump".
@@ -657,6 +670,10 @@ class Lpar(BaseResource):
             an additional acceptable end status when `wait_for_completion` is
             set.
 
+          force (bool):
+            Boolean controlling whether this operation is permitted when the
+            LPAR is in the "operating" status.
+
         Returns:
 
           `None` or :class:`~zhmcclient.Job`:
@@ -694,6 +711,8 @@ class Lpar(BaseResource):
                 boot_record_logical_block_address
         if os_ipl_token is not None:
             body['os-ipl-token'] = os_ipl_token
+        if force:
+            body['force'] = force
         result = self.manager.session.post(
             self.uri + '/operations/scsi-dump',
             body,
