@@ -73,8 +73,13 @@ import six
 
 from ._manager import BaseManager
 from ._resource import BaseResource
+from ._cpc import Cpc
+from ._lpar import Lpar
+from ._partition import Partition
+from ._adapter import Adapter
+from ._nic import Nic
 from ._logging import logged_api_call
-from ._exceptions import NotFound
+from ._exceptions import NotFound, MetricsResourceNotFound
 from ._utils import datetime_from_timestamp, repr_list
 
 __all__ = ['MetricsContextManager', 'MetricsContext', 'MetricGroupDefinition',
@@ -166,6 +171,7 @@ class MetricsContextManager(BaseManager):
 
     @logged_api_call
     def list(self, full_properties=False):
+        # pylint: disable=arguments-differ
         """
         List the :term:`Metrics Context` resources that were created through
         this manager object.
@@ -344,6 +350,7 @@ class MetricsContext(BaseResource):
           :exc:`~zhmcclient.AuthError`
           :exc:`~zhmcclient.ConnectionError`
         """
+        # pylint: disable=protected-access
         self.manager.session.delete(self.uri)
         self.manager._metrics_contexts.remove(self)
 
@@ -408,6 +415,7 @@ class MetricDefinition(_MetricDefinitionTuple):
     """
 
     def __new__(cls, index, name, type, unit):
+        # pylint: disable=redefined-builtin
         """
         Parameters:
 
@@ -489,8 +497,11 @@ def _metric_value(value_str, metric_type):
         try:
             return metric_type(value_str)
         except ValueError:
-            raise ValueError("Invalid {} metric value: {!r}".
-                             format(metric_type.__class__.__name__, value_str))
+            new_exc = ValueError(
+                "Invalid {} metric value: {!r}".
+                format(metric_type.__class__.__name__, value_str))
+            new_exc.__cause__ = None
+            raise new_exc  # ValueError
     elif metric_type is six.text_type:
         # In Python 3, decode('unicode_escape) requires bytes, so we need
         # to encode to bytes. This also works in Python 2.
@@ -500,11 +511,9 @@ def _metric_value(value_str, metric_type):
         lower_str = value_str.lower()
         if lower_str == 'true':
             return True
-        elif lower_str == 'false':
+        if lower_str == 'false':
             return False
-        else:
-            raise ValueError("Invalid boolean metric value: {!r}".
-                             format(value_str))
+        raise ValueError("Invalid boolean metric value: {!r}".format(value_str))
 
 
 def _metric_unit_from_name(metric_name):
@@ -850,8 +859,9 @@ class MetricObjectValues(object):
 
         Raises:
 
-          :exc:`~zhmcclient.NotFound`: No resource found for this URI in the
-            management scope of the HMC.
+          :exc:`~zhmcclient.MetricsResourceNotFound`: The resource for the
+          resource URI of the resource these metric values apply to, was not
+          found on the HMC.
         """
         if self._resource is not None:
             return self._resource
@@ -860,10 +870,21 @@ class MetricObjectValues(object):
         resource_uri = self.resource_uri
 
         if resource_class == 'cpc':
+            cpc_managers = [self.client]
             filter_args = {'object-uri': resource_uri}
-            resource = self.client.cpcs.find(**filter_args)
+            try:
+                resource = self.client.cpcs.find(**filter_args)
+            except NotFound:
+                raise MetricsResourceNotFound(
+                    "{} with URI {} not found on HMC {}".
+                    format(resource_class, resource_uri,
+                           self.client.session.host),
+                    Cpc, cpc_managers)
         elif resource_class == 'logical-partition':
-            for cpc in self.client.cpcs.list():
+            lpar_managers = []
+            cpc_list = self.client.cpcs.list()
+            for cpc in cpc_list:
+                lpar_managers.append(cpc.lpars)
                 try:
                     filter_args = {'object-uri': resource_uri}
                     resource = cpc.lpars.find(**filter_args)
@@ -871,9 +892,16 @@ class MetricObjectValues(object):
                 except NotFound:
                     pass  # Try next CPC
             else:
-                raise
+                raise MetricsResourceNotFound(
+                    "{} with URI {} not found in CPCs {}".
+                    format(resource_class, resource_uri,
+                           ', '.join([cpc.name for cpc in cpc_list])),
+                    Lpar, lpar_managers)
         elif resource_class == 'partition':
-            for cpc in self.client.cpcs.list():
+            cpc_list = self.client.cpcs.list()
+            partition_managers = []
+            for cpc in cpc_list:
+                partition_managers.append(cpc.partitions)
                 try:
                     filter_args = {'object-uri': resource_uri}
                     resource = cpc.partitions.find(**filter_args)
@@ -881,9 +909,16 @@ class MetricObjectValues(object):
                 except NotFound:
                     pass  # Try next CPC
             else:
-                raise
+                raise MetricsResourceNotFound(
+                    "{} with URI {} not found in CPCs {}".
+                    format(resource_class, resource_uri,
+                           ', '.join([cpc.name for cpc in cpc_list])),
+                    Partition, partition_managers)
         elif resource_class == 'adapter':
-            for cpc in self.client.cpcs.list():
+            cpc_list = self.client.cpcs.list()
+            adapter_managers = []
+            for cpc in cpc_list:
+                adapter_managers.append(cpc.adapters)
                 try:
                     filter_args = {'object-uri': resource_uri}
                     resource = cpc.adapters.find(**filter_args)
@@ -891,11 +926,18 @@ class MetricObjectValues(object):
                 except NotFound:
                     pass  # Try next CPC
             else:
-                raise
+                raise MetricsResourceNotFound(
+                    "{} with URI {} not found in CPCs {}".
+                    format(resource_class, resource_uri,
+                           ', '.join([cpc.name for cpc in cpc_list])),
+                    Adapter, adapter_managers)
         elif resource_class == 'nic':
-            for cpc in self.client.cpcs.list():
+            cpc_list = self.client.cpcs.list()
+            nic_managers = []
+            for cpc in cpc_list:
                 found = False
                 for partition in cpc.partitions.list():
+                    nic_managers.append(partition.nics)
                     try:
                         filter_args = {'element-uri': resource_uri}
                         resource = partition.nics.find(**filter_args)
@@ -906,7 +948,11 @@ class MetricObjectValues(object):
                 if found:
                     break
             else:
-                raise
+                raise MetricsResourceNotFound(
+                    "{} with URI {} not found in the partitions of CPCs {}".
+                    format(resource_class, resource_uri,
+                           ', '.join([cpc.name for cpc in cpc_list])),
+                    Nic, nic_managers)
         else:
             raise ValueError(
                 "Invalid resource class: {!r}".format(resource_class))
