@@ -220,7 +220,7 @@ class UserPatternManager(BaseManager):
         body = {
             'user-pattern-uris': [up.uri for up in user_patterns]
         }
-        self.manager.session.post(
+        self.session.post(
             '/api/console/operations/reorder-user-patterns',
             body=body)
 
@@ -269,6 +269,7 @@ class UserPattern(BaseResource):
           :exc:`~zhmcclient.AuthError`
           :exc:`~zhmcclient.ConnectionError`
         """
+        # pylint: disable=protected-access
         self.manager.session.delete(self.uri)
         self.manager._name_uri_cache.delete(
             self.properties.get(self.manager._name_prop, None))
@@ -297,6 +298,7 @@ class UserPattern(BaseResource):
           :exc:`~zhmcclient.AuthError`
           :exc:`~zhmcclient.ConnectionError`
         """
+        # pylint: disable=protected-access
         self.manager.session.post(self.uri, body=properties)
         is_rename = self.manager._name_prop in properties
         if is_rename:
