@@ -27,7 +27,7 @@ __all__ = ['__version__']
 #:
 #: * "M.N.P.dev1": A not yet released version M.N.P
 #: * "M.N.P": A released version M.N.P
-__version__ = '0.26.1'
+__version__ = '0.30.0'
 
 # Check supported Python versions
 # Keep these Python versions in sync with:
@@ -37,5 +37,5 @@ _PYTHON_M = sys.version_info[0]
 _PYTHON_N = sys.version_info[1]
 if _PYTHON_M == 2 and _PYTHON_N < 7:
     raise RuntimeError('On Python 2, zhcmclient requires Python 2.7')
-elif _PYTHON_M == 3 and _PYTHON_N < 4:
+if _PYTHON_M == 3 and _PYTHON_N < 4:
     raise RuntimeError('On Python 3, zhmcclient requires Python 3.4 or higher')
